@@ -113,7 +113,7 @@
 # define DI_LL "l"
 # define DI_LLu "lu"
 #endif
-#if _siz_long_double >= 8
+#if _siz_long_double >= 8 && _printf_long_double
   typedef long double _print_size_t;
 # define DI_Lf "Lf"
 #else
@@ -220,6 +220,7 @@ typedef struct {
     unsigned int    posix_compat;
     unsigned int    quota_check;
     unsigned int    csv_output;
+    unsigned int    csv_tabs;
     unsigned int    excludeLoopback;
     unsigned int    printTotals;
     unsigned int    printDebugHeader;
@@ -237,7 +238,7 @@ typedef struct {
     Size_t       maxSpecialString;
     Size_t       maxTypeString;
     Size_t       width;
-    char         *dispBlockLabel;
+    const char   *dispBlockLabel;
     char         blockFormat [15];
     char         blockFormatNR [15];   /* no radix */
     char         inodeFormat [15];
