@@ -673,7 +673,6 @@ di_getDiskEntries (diskInfo, diCount)
 #   define MNT_NUMTYPES (sizeof(mnt_names)/sizeof(char *))
 #  endif
 # endif
-# define DI_UNKNOWN_FSTYPE       "(%.2d)?"
 
 int
 # if _proto_stdc
@@ -708,7 +707,6 @@ di_getDiskEntries (diskInfo, diCount)
 
     *diCount = count;
     *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) * (Size_t) count);
-
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
@@ -793,6 +791,7 @@ di_getDiskEntries (diskInfo, diCount)
         sysfs (GETFSTYP, sp->f_type, diptr->fsType);
 #  else
 #   if _dcl_mnt_names && _mem_struct_statfs_f_type
+#    define DI_UNKNOWN_FSTYPE       "(%.2d)?"
         fstype = sp->f_type;
         if ((fstype >= 0) && (fstype < MNT_NUMTYPES))
         {
@@ -818,8 +817,6 @@ di_getDiskEntries (diskInfo, diCount)
     && ! _lib_getfsstat \
     && ! _lib_getvfsstat
 
-# define DI_UNKNOWN_FSTYPE       "(%.2d)?"
-
 /*
  * di_getDiskEntries
  *
@@ -1047,6 +1044,8 @@ di_getDiskEntries (diskInfo, diCount)
 #  if _mem_struct_statfs_f_fstypename
         strncpy (diptr->fsType, mntbufp [idx].f_fstypename, DI_TYPE_LEN);
 #  else
+#   define DI_UNKNOWN_FSTYPE       "(%.2d)?"
+
             /* could use getvfsbytype here... */
         if ((fstype >= 0) && (fstype < MNT_NUMTYPES))
         {
@@ -1450,7 +1449,7 @@ di_getDiskEntries (diskInfo, diCount)
 
         /* <num> vmount structs returned in vmbuf */
     *diCount = num;
-    *diskInfo = (diDiskInfo_t *) calloc (sizeof (diDiskInfo_t),
+    *diskInfo = (diDiskInfo_t *) malloc (sizeof (diDiskInfo_t) *
         (Size_t) *diCount);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
@@ -1559,7 +1558,7 @@ di_getDiskEntries (diskInfo, diCount)
     *diCount = rc / BYTES_PER_LOGICAL_DRIVE;
 
     *diskInfo = (diDiskInfo_t *)
-        calloc (sizeof (diDiskInfo_t), (Size_t) *diCount);
+        malloc (sizeof (diDiskInfo_t) * (Size_t) *diCount);
     if (*diskInfo == (diDiskInfo_t *) NULL)
     {
         fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
