@@ -1,7 +1,4 @@
 /*
- * $Id$
- * $Source$
- *
  * Copyright 1994-2013 Brad Lanam, Walnut Creek, CA
  */
 
@@ -37,6 +34,9 @@
 #if _hdr_wchar
 # include <wchar.h>
 #endif
+#if _use_mcheck
+# include <mcheck.h>
+#endif
 
 extern int debug;
 
@@ -72,6 +72,8 @@ extern int debug;
 #define DI_SORT_NONE            'n'
 #define DI_SORT_MOUNT           'm'
 #define DI_SORT_SPECIAL         's'
+#define DI_SORT_TOTAL           'T'
+#define DI_SORT_FREE            'f'
 #define DI_SORT_AVAIL           'a'
 #define DI_SORT_REVERSE         'r'
 #define DI_SORT_TYPE            't'
@@ -87,8 +89,8 @@ typedef struct
     _print_size_t   low;
     _print_size_t   high;
     _print_size_t   dbs;        /* display block size */
-    char            *format;
-    char            *suffix;
+    const char      *format;
+    const char      *suffix;
 } sizeTable_t;
 
 static sizeTable_t sizeTable [] =
@@ -115,11 +117,14 @@ static void getMaxFormatLengths _((diData_t *));
 static int  diCompare           _((const diOptions_t *, const diDiskInfo_t *, unsigned int, unsigned int));
 static int  findDispSize        _((_print_size_t));
 static Size_t istrlen           _((const char *));
-static void printInfo           _((diDiskInfo_t *, diOptions_t *, diOutput_t *));
-static void printSpace          _((const diOptions_t *, const diOutput_t *, _fs_size_t, int));
-static void processTitles       _((diOptions_t *, diOutput_t *));
-static void printPerc           _((_fs_size_t, _fs_size_t, const char *));
+static char *printInfo          _((diDiskInfo_t *, diOptions_t *, diOutput_t *));
+static char *printSpace         _((const diOptions_t *, const diOutput_t *, _fs_size_t, int));
+static char *processTitles      _((diOptions_t *, diOutput_t *));
+static char *printPerc          _((_fs_size_t, _fs_size_t, const char *));
 static void initSizeTable       _((diOptions_t *, diOutput_t *));
+static void appendFormatStr     _((char *, const char *, char **, Size_t *, Size_t *));
+static void appendFormatVal     _((char *, _fs_size_t, char **, Size_t *, Size_t *));
+static void append              _((const char *, char **, Size_t *, Size_t *));
 
 #if defined (__cplusplus) || defined (c_plusplus)
   }
@@ -136,7 +141,7 @@ static void initSizeTable       _((diOptions_t *, diOutput_t *));
  *
  */
 
-void
+char *
 #if _proto_stdc
 printDiskInfo (diData_t *diData)
 #else
@@ -152,7 +157,14 @@ printDiskInfo (diData)
     Size_t              lastpoollen = { 0 };
     int                 inpool = { FALSE };
     diOutput_t          *diout;
+    char                *out;
+    Size_t              outlen;
+    Size_t              outcurrlen;
+    char                *tout;
 
+    out = (char *) NULL;
+    outlen = 0;
+    outcurrlen = 0;
     lastpool[0] = '\0';
     diopts = &diData->options;
     diout = &diData->output;
@@ -166,7 +178,11 @@ printDiskInfo (diData)
     }
 
     getMaxFormatLengths (diData);
-    processTitles (diopts, diout);
+    tout = processTitles (diopts, diout);
+    if (diopts->printHeader) {
+      append (tout, &out, &outcurrlen, &outlen);
+    }
+    free (tout);
 
     if (diopts->dispBlockSize == DI_DISP_HR ||
         diopts->dispBlockSize == DI_DISP_HR_2)
@@ -188,8 +204,13 @@ printDiskInfo (diData)
       if (diopts->csv_output) {
         Snprintf1 (diout->blockFormatNR, sizeof (diout->blockFormatNR),
             "%%.0%s%%s", DI_Lf);
-        Snprintf1 (diout->blockFormat, sizeof (diout->blockFormat),
-            "\"%%.1%s%%s\"", DI_Lf);
+        if (diopts->csv_tabs) {
+          Snprintf1 (diout->blockFormat, sizeof (diout->blockFormat),
+              "%%.1%s%%s", DI_Lf);
+        } else {
+          Snprintf1 (diout->blockFormat, sizeof (diout->blockFormat),
+              "\"%%.1%s%%s\"", DI_Lf);
+        }
       } else {
         Snprintf2 (diout->blockFormatNR, sizeof (diout->blockFormatNR),
             "%%%d.0%s%%s", (int) diout->width, DI_Lf);
@@ -215,9 +236,6 @@ printDiskInfo (diData)
     diskInfo = diData->diskInfo;
     if (diopts->printTotals)
     {
-        int                  allocated;
-
-        allocated = FALSE;
         if (diData->haspooledfs && ! diData->totsorted)
         {
           char tempSortType [DI_SORT_MAX + 1];
@@ -302,13 +320,18 @@ printDiskInfo (diData)
         continue;
       }
 
-      printInfo (dinfo, diopts, diout);
+      tout = printInfo (dinfo, diopts, diout);
+      append (tout, &out, &outcurrlen, &outlen);
+      free (tout);
     }
 
     if (diopts->printTotals)
     {
-      printInfo (&totals, diopts, diout);
+      tout = printInfo (&totals, diopts, diout);
+      append (tout, &out, &outcurrlen, &outlen);
+      free (tout);
     }
+    return out;
 }
 
 /*
@@ -383,6 +406,73 @@ getPrintFlagText (pf)
 }
 
 
+static void
+#if _proto_stdc
+appendFormatStr (char *fmt, const char *val, char **ptr, Size_t *clen, Size_t *len)
+#else
+appendFormatStr (fmt, val, ptr, clen, len)
+  char *fmt;
+  const char *val;
+  char **ptr;
+  Size_t *clen;
+  Size_t *len;
+#endif
+{
+  char          tdata [1024];
+
+  Snprintf1 (tdata, sizeof(tdata), fmt, val);
+  append (tdata, ptr, clen, len);
+}
+
+static void
+#if _proto_stdc
+appendFormatVal (char *fmt, _fs_size_t val, char **ptr, Size_t *clen, Size_t *len)
+#else
+appendFormatVal (fmt, val, ptr, clen, len)
+  char *fmt;
+  _fs_size_t val;
+  char **ptr;
+  Size_t *clen;
+  Size_t *len;
+#endif
+{
+  char          tdata [1024];
+
+  Snprintf1 (tdata, sizeof(tdata), fmt, val);
+  append (tdata, ptr, clen, len);
+}
+
+static void
+#if _proto_stdc
+append (const char *val, char **ptr, Size_t *clen, Size_t *len)
+#else
+append (val, ptr, len)
+  const char *val;
+  char      **ptr;
+  Size_t    *clen;
+  Size_t    *len;
+#endif
+{
+  Size_t    vlen;
+  Size_t    bumplen;
+  Size_t    nlen;
+
+  if (val == (char *) NULL) {
+    return;
+  }
+
+  bumplen = 100;
+  vlen = strlen (val);
+  if (*clen + vlen >= *len) {
+    nlen = vlen + *clen + 1;
+    *ptr = realloc (*ptr, nlen);
+    memset (*ptr+*clen, 0, nlen-*clen);
+    *len = nlen;
+  }
+  strcat (*ptr, val);
+  *clen += vlen;
+}
+
 /*
  * printInfo
  *
@@ -391,7 +481,7 @@ getPrintFlagText (pf)
  *
  */
 
-static void
+static char *
 #if _proto_stdc
 printInfo (diDiskInfo_t *diskInfo, diOptions_t *diopts, diOutput_t *diout)
 #else
@@ -412,6 +502,16 @@ printInfo (diskInfo, diopts, diout)
     static char         percFormat [15];
     static int          percInit = FALSE;
     int                 first;
+    char                ttext[2];
+    char                *out;
+    char                *tout;
+    const char          *t;
+    Size_t              outlen;
+    Size_t              outcurrlen;
+
+    out = (char *) NULL;
+    outlen = 0;
+    outcurrlen = 0;
 
     first = TRUE;
     if (! percInit) {
@@ -505,7 +605,11 @@ printInfo (diskInfo, diopts, diout)
       }
       if (valid && diopts->csv_output) {
         if (! first) {
-          printf (",");
+          t = ",";
+          if (diopts->csv_tabs) {
+            t = "	"; /* tab here */
+          }
+          append (t, &out, &outcurrlen, &outlen);
         }
         first = FALSE;
       }
@@ -515,46 +619,52 @@ printInfo (diskInfo, diopts, diout)
         case DI_FMT_MOUNT:
         case DI_FMT_MOUNT_FULL:
         {
-          printf (diout->mountFormat, diskInfo->name);
+          appendFormatStr (diout->mountFormat, diskInfo->name, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BTOT:
         {
-          printSpace (diopts, diout, diskInfo->totalSpace, idx);
+          tout = printSpace (diopts, diout, diskInfo->totalSpace, idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BTOT_AVAIL:
         {
-          printSpace (diopts, diout, diskInfo->totalSpace -
+          tout = printSpace (diopts, diout, diskInfo->totalSpace -
               (diskInfo->freeSpace - diskInfo->availSpace), idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BUSED:
         {
-          printSpace (diopts, diout,
+          tout = printSpace (diopts, diout,
               diskInfo->totalSpace - diskInfo->freeSpace, idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BCUSED:
         {
-          printSpace (diopts, diout,
+          tout = printSpace (diopts, diout,
               diskInfo->totalSpace - diskInfo->availSpace, idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BFREE:
         {
-          printSpace (diopts, diout, diskInfo->freeSpace, idx);
+          tout = printSpace (diopts, diout, diskInfo->freeSpace, idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_BAVAIL:
         {
-          printSpace (diopts, diout, diskInfo->availSpace, idx);
+          tout = printSpace (diopts, diout, diskInfo->availSpace, idx);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -562,7 +672,8 @@ printInfo (diskInfo, diopts, diout)
         {
           used = diskInfo->totalSpace - diskInfo->availSpace;
           totAvail = diskInfo->totalSpace;
-          printPerc (used, totAvail, percFormat);
+          tout = printPerc (used, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -570,7 +681,8 @@ printInfo (diskInfo, diopts, diout)
         {
           used = diskInfo->totalSpace - diskInfo->freeSpace;
           totAvail = diskInfo->totalSpace;
-          printPerc (used, totAvail, percFormat);
+          tout = printPerc (used, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -579,7 +691,8 @@ printInfo (diskInfo, diopts, diout)
           used = diskInfo->totalSpace - diskInfo->freeSpace;
           totAvail = diskInfo->totalSpace -
                   (diskInfo->freeSpace - diskInfo->availSpace);
-          printPerc (used, totAvail, percFormat);
+          tout = printPerc (used, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -588,7 +701,8 @@ printInfo (diskInfo, diopts, diout)
           _fs_size_t          bfree;
           bfree = diskInfo->availSpace;
           totAvail = diskInfo->totalSpace;
-          printPerc (bfree, totAvail, percFormat);
+          tout = printPerc (bfree, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -597,25 +711,30 @@ printInfo (diskInfo, diopts, diout)
           _fs_size_t          bfree;
           bfree = diskInfo->freeSpace;
           totAvail = diskInfo->totalSpace;
-          printPerc (bfree, totAvail, percFormat);
+          tout = printPerc (bfree, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_ITOT:
         {
-          printf (diout->inodeFormat, diskInfo->totalInodes);
+          appendFormatVal (diout->inodeFormat, diskInfo->totalInodes,
+              &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_IUSED:
         {
-          printf (diout->inodeFormat, diskInfo->totalInodes - diskInfo->freeInodes);
+          appendFormatVal (diout->inodeFormat,
+              diskInfo->totalInodes - diskInfo->freeInodes,
+              &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_IFREE:
         {
-          printf (diout->inodeFormat, diskInfo->freeInodes);
+          appendFormatVal (diout->inodeFormat, diskInfo->freeInodes,
+              &out, &outcurrlen, &outlen);
           break;
         }
 
@@ -623,32 +742,34 @@ printInfo (diskInfo, diopts, diout)
         {
           used = diskInfo->totalInodes - diskInfo->availInodes;
           totAvail = diskInfo->totalInodes;
-          printPerc (used, totAvail, percFormat);
+          tout = printPerc (used, totAvail, percFormat);
+          append (tout, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_SPECIAL:
         case DI_FMT_SPECIAL_FULL:
         {
-          printf (diout->specialFormat, diskInfo->special);
+          appendFormatStr (diout->specialFormat, diskInfo->special,
+              &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_TYPE:
         case DI_FMT_TYPE_FULL:
         {
-          printf (diout->typeFormat, diskInfo->fsType);
+          appendFormatStr (diout->typeFormat, diskInfo->fsType, &out, &outcurrlen, &outlen);
           break;
         }
 
         case DI_FMT_MOUNT_OPTIONS:
         {
-          if (diopts->csv_output) {
-            printf ("\"");
+          if (diopts->csv_output && ! diopts->csv_tabs) {
+            append ("\"", &out, &outcurrlen, &outlen);
           }
-          printf (diout->optFormat, diskInfo->options);
-          if (diopts->csv_output) {
-            printf ("\"");
+          appendFormatStr (diout->optFormat, diskInfo->options, &out, &outcurrlen, &outlen);
+          if (diopts->csv_output && ! diopts->csv_tabs) {
+            append ("\"", &out, &outcurrlen, &outlen);
           }
           break;
         }
@@ -660,7 +781,9 @@ printInfo (diskInfo, diopts, diout)
 
         default:
         {
-          printf ("%c", *ptr);
+          ttext[0] = *ptr;
+          ttext[1] = '\0';
+          append (ttext, &out, &outcurrlen, &outlen);
           break;
         }
       }
@@ -668,14 +791,17 @@ printInfo (diskInfo, diopts, diout)
       ++ptr;
       if (! diopts->csv_output && *ptr && valid)
       {
-        printf (" ");
+        append (" ", &out, &outcurrlen, &outlen);
       }
     }
 
-    printf ("\n");
+    if (outcurrlen > 0) {
+      append ("\n", &out, &outcurrlen, &outlen);
+    }
+    return out;
 }
 
-static void
+static char *
 #if _proto_stdc
 printSpace (const diOptions_t *diopts, const diOutput_t *diout,
              _fs_size_t usage, int idx)
@@ -690,8 +816,9 @@ printSpace (diopts, diout, usage, idx)
     _print_size_t   tdbs;
     _print_size_t   mult;
     _print_size_t   temp;
-    char            *suffix;
+    const char      *suffix;
     const char      *format;
+    static char     tdata [1024];
 
 
     suffix = "";
@@ -720,7 +847,8 @@ printSpace (diopts, diout, usage, idx)
     }
 
     mult = 1.0 / tdbs;
-    printf (format, (_print_size_t) usage * mult, suffix);
+    Snprintf2 (tdata, sizeof(tdata), format, (_print_size_t) usage * mult, suffix);
+    return tdata;
 }
 
 
@@ -797,7 +925,7 @@ addTotals (diskInfo, totals, inpool)
  *
  */
 
-static void
+static char *
 #if _proto_stdc
 processTitles (diOptions_t *diopts, diOutput_t *diout)
 #else
@@ -811,14 +939,19 @@ processTitles (diopts, diout)
     Size_t          wlen;
     Size_t          *wlenptr;
     int             justification;
-    char            *pstr = { "" };
+    const char      *pstr = { "" };
     char            *fstr;
     Size_t          maxsize;
     char            tformat [30];
-    char            ttitle [2];
+    char            ttext [2];
     int             first;
+    char            *out;
+    Size_t          outcurrlen;
+    Size_t          outlen;
 
-
+    out = (char *) NULL;
+    outlen = 0;
+    outcurrlen = 0;
     first = TRUE;
     if (diopts->printDebugHeader)
     {
@@ -1036,9 +1169,9 @@ processTitles (diopts, diout)
 
             default:
             {
-              if (diopts->printHeader) {
-                printf ("%c", *ptr);
-              }
+              ttext[0] = *ptr;
+              ttext[1] = '\0';
+              append (ttext, &out, &outcurrlen, &outlen);
               valid = FALSE;
               break;
             }
@@ -1062,24 +1195,31 @@ processTitles (diopts, diout)
           Snprintf3 (tformat, sizeof (tformat), "%%%s%d.%ds",
               jstr, (int) tlen, (int) tlen);
 
-          if (diopts->printHeader) {
-            if (diopts->csv_output) {
-              if (! first) {
-                printf (",");
+          if (diopts->csv_output) {
+            if (! first) {
+              if (diopts->csv_tabs) {
+                append ("	", &out, &outcurrlen, &outlen); /* tab here */
+              } else {
+                append (",", &out, &outcurrlen, &outlen);
               }
-              first = FALSE;
-            }
-            if (diopts->csv_output) {
-              ttitle[0] = *ptr;
-              ttitle[1] = '\0';
-              printf ("%s", ttitle);
-            } else {
-              printf (tformat, pstr);
             }
+            first = FALSE;
           }
+          if (diopts->csv_output) {
+            ttext[0] = *ptr;
+            ttext[1] = '\0';
+            append (ttext, &out, &outcurrlen, &outlen);
+          } else {
+            appendFormatStr (tformat, pstr, &out, &outcurrlen, &outlen);
+          }
+
           if (fstr != (char *) NULL) {
             if (diopts->csv_output) {
-              strncpy (tformat, "\"%s\"", sizeof (tformat));
+              if (diopts->csv_tabs) {
+                strncpy (tformat, "%s", sizeof (tformat));
+              } else {
+                strncpy (tformat, "\"%s\"", sizeof (tformat));
+              }
             }
             if (tlen != len) {
               if (! diopts->csv_output) {
@@ -1095,17 +1235,16 @@ processTitles (diopts, diout)
         }
 
         ++ptr;
-        if (diopts->printHeader) {
-          if (! diopts->csv_output && *ptr && valid)
-          {
-              printf (" ");
-          }
+        if (! diopts->csv_output && *ptr && valid)
+        {
+          append (" ", &out, &outcurrlen, &outlen);
         }
     }
 
-    if (diopts->printHeader) {
-      printf ("\n");
+    if (outcurrlen > 0) {
+      append ("\n", &out, &outcurrlen, &outlen);
     }
+    return out;
 }
 
 /*
@@ -1115,7 +1254,7 @@ processTitles (diopts, diout)
  *
  */
 
-static void
+static char *
 #if _proto_stdc
 printPerc (_fs_size_t used, _fs_size_t totAvail, const char *format)
 #else
@@ -1125,19 +1264,19 @@ printPerc (used, totAvail, format)
     const char      *format;
 #endif
 {
-    _print_size_t      perc;
+    _print_size_t   perc;
+    static char     tdata [1024];
 
-
-    if (totAvail > 0L)
-    {
+    if (totAvail > 0L) {
         perc = (_print_size_t) used / (_print_size_t) totAvail;
         perc *= 100.0;
     }
-    else
-    {
+    else {
         perc = 0.0;
     }
-    printf (format, perc);
+
+    Snprintf1 (tdata, sizeof(tdata), format, perc);
+    return tdata;
 }
 
 
@@ -1204,10 +1343,30 @@ diCompare (diopts, data, idx1, idx2)
         }
 
         case DI_SORT_AVAIL:
+        case DI_SORT_FREE:
+        case DI_SORT_TOTAL:
         {
           _fs_size_t    temp;
 
-          temp = (d1->availSpace - d2->availSpace);
+          temp = 0;
+          switch (*ptr) {
+            case DI_SORT_AVAIL:
+            {
+              temp = (d1->availSpace - d2->availSpace);
+              break;
+            }
+            case DI_SORT_FREE:
+            {
+              temp = (d1->freeSpace - d2->freeSpace);
+              break;
+            }
+            case DI_SORT_TOTAL:
+            {
+              temp = (d1->totalSpace - d2->totalSpace);
+              break;
+            }
+          }
+
           if (temp == 0) {
             rc = 0;
           } else {
