@@ -43,6 +43,9 @@
 # define _DI_INC_SYS_TYPES_H
 # include <sys/types.h>
 #endif
+#if _use_mcheck
+# include <mcheck.h>
+#endif
 
 typedef struct {
     Size_t      optionlen;
@@ -174,13 +177,13 @@ getoption_value (info, opt)
 static int
 #if _proto_stdc
 dooptchecks (getoptn_info_t *info, getoptn_opt_t *opt,
-     getoptn_optinfo_t *optinfo, char *tag, Size_t sz)
+     getoptn_optinfo_t *optinfo, const char *tag, Size_t sz)
 #else
 dooptchecks (info, opt, optinfo, tag, sz)
     getoptn_info_t      *info;
     getoptn_opt_t       *opt;
     getoptn_optinfo_t   *optinfo;
-    char                *tag;
+    const char          *tag;
     Size_t              sz;
 #endif
 {
@@ -271,7 +274,7 @@ process_opt (info, opt, optinfo)
       return;
     }
     v = (const char **) opt->valptr;
-    *v = ptr;
+    *v = strdup (ptr); /* memory leak (one time) */
   } else if (opt->option_type == GETOPTN_FUNC_BOOL) {
     getoptn_func_bool_t f;
     if (opt->value2 == (void *) NULL) {
