@@ -30,11 +30,6 @@ for f in $@; do
     "-e")
       doecho=T
       ;;
-    "-c")
-      shift
-      comp=$1
-      shift
-      ;;
     "--")
       ;;
     "-L")
@@ -93,8 +88,8 @@ done
 #fi
 
 if [ $grc -eq 0 ]; then
-  dosubst libnm '${SHLIB_EXT}$' '' '^lib' ''
-  libfnm=lib${libnm}${SHLIB_EXT}
+  dosubst libnm '${SHLIB_EXT}$' ''
+  libfnm=${libnm}${SHLIB_EXT}
   shrunpath=""
   if [ "${libs}" != "" -a "${SHRUNPATH}" != "" ]; then
     shrunpath=${libpath}
@@ -105,7 +100,7 @@ if [ $grc -eq 0 ]; then
     shlibpath=${libpath}
     dosubst shlibpath '^:' '-L'
   fi
-  cmd="${comp} ${LDFLAGS} ${SHLDFLAGS} -o $libfnm ${shrunpath} ${shlibpath} ${objects} ${libs}"
+  cmd="${comp} ${SHLDFLAGS} -o $libfnm ${shrunpath} ${shlibpath} ${objects} ${libs}"
   if [ "$doecho" = "T" ]; then
     echo $cmd
   fi
