@@ -55,8 +55,8 @@ if [ "$arcmd" = "" ]; then
 fi
 
 if [ $grc -eq 0 ]; then
-  dosubst libnm '${SHLIB_EXT}$' '' '^lib' ''
-  libfnm=lib${libnm}.a
+  dosubst libnm '${SHLIB_EXT}$' ''
+  libfnm=${libnm}.a
   # for really old systems...
   if [ "$ranlibcmd" = "" -a "$lordercmd" != "" -a "$tsortcmd" != "" ]; then
     objects=`$lordercmd ${objects} | $tsortcmd`
