@@ -13,25 +13,28 @@
 #
 #   The following code is present in the final config.h file:
 #
-#       #if ! _key_void
-#       # define void int
-#       #endif
-#       #if ! _key_const
-#       # define const
-#       #endif
-#       #if ! _key_void || ! _param_void_star
-#         typedef char *_pvoid;
-#       #else
-#         typedef void *_pvoid;
-#       #endif
-#
-#       #ifndef _
-#       # if _proto_stdc
-#       #  define _(args) args
-#       # else
-#       #  define _(args) ()
+#       #ifndef __MKC_STANDARD_DEFS
+#       # define __MKC_STANDARD_DEFS 1
+#       # if ! _key_void
+#       #  define void int
 #       # endif
-#       #endif
+#       # if ! _key_const
+#       #  define const
+#       # endif
+#       # if ! _key_void || ! _param_void_star
+#          typedef char *_pvoid;
+#       # else
+#          typedef void *_pvoid;
+#       # endif
+#
+#       # ifndef _
+#       #  if _proto_stdc
+#       #   define _(args) args
+#       #  else
+#       #   define _(args) ()
+#       #  endif
+#       # endif
+#       #endif /* __MKC_STANDARD_DEFS */
 #
 
 #
@@ -99,25 +102,28 @@ preconfigfile () {
 stdconfigfile () {
   pc_configfile=$1
   echo '
-#if ! _key_void
-# define void int
-#endif
-#if ! _key_void || ! _param_void_star
-  typedef char *_pvoid;
-#else
-  typedef void *_pvoid;
-#endif
-#if ! _key_const
-# define const
-#endif
-
-#ifndef _
-# if _proto_stdc
-#  define _(args) args
-# else
-#  define _(args) ()
+#ifndef __MKC_STANDARD_DEFS
+# define __MKC_STANDARD_DEFS 1
+# if ! _key_void
+#  define void int
 # endif
-#endif
+# if ! _key_void || ! _param_void_star
+   typedef char *_pvoid;
+# else
+   typedef void *_pvoid;
+# endif
+# if ! _key_const
+#  define const
+# endif
+
+# ifndef _
+#  if _proto_stdc
+#   define _(args) args
+#  else
+#   define _(args) ()
+#  endif
+# endif
+#endif /* __MKC_STANDARD_DEFS */
 '
 }
 
@@ -338,6 +344,34 @@ tparamvs (ptr)
   do_c_check_compile ${name} "${code}" all
 }
 
+check_printf_long_double () {
+  name="_printf_long_double"
+
+  printlabel $name "printf: long double printable"
+  checkcache ${_MKCONFIG_PREFIX} $name
+  if [ $rc -eq 0 ]; then return; fi
+
+  code="int main (int argc, char *argv[]) {
+long double a;
+long double b;
+char t[40];
+a = 1.0;
+b = 2.0;
+a = a / b;
+sprintf (t, \"%.1Lf\", a);
+if (strcmp(t,\"0.5\") == 0) {
+return (0);
+}
+return (1);
+}"
+
+  _c_chk_run "$name" "$code" all
+  rc=$?
+  if [ $rc -eq 0 ]; then trc=1; else trc=0; fi
+  setdata ${_MKCONFIG_PREFIX} ${name} ${trc}
+  printyesno $name $trc
+}
+
 check_member () {
   shift
   struct=$1
@@ -458,8 +492,27 @@ check_args () {
   ccount=0
 
   oldprecc="${precc}"
+
+  if [ "$_have_variadic" = "" ]; then
+    precc=""
+    code="#define testvariadic(...)"
+    _c_chk_cpp _args_testvariadic "$code" all
+    rc=$?
+    _have_variadic=F
+    if [ $rc -eq 0 ]; then
+      _have_variadic=T
+    fi
+  fi
+
+  asmdef="#define __asm__(a)"
+  if [ $_have_variadic = T ]; then
+    asmdef="#define __asm__(...)"
+  fi
+
+  precc="$oldprecc"
   doappend precc "/* get rid of most gcc-isms */
-#define __asm__(a)
+#define __asm(a)
+$asmdef
 #define __attribute__(a)
 #define __nonnull__(a,b)
 #define __restrict
