@@ -143,7 +143,7 @@ check_using_clang () {
 
   printlabel _MKCONFIG_USING_CLANG "Using clang"
 
-  # check for gcc...
+  # check for clang...
   ${CC} -v 2>&1 | grep 'clang version' > /dev/null 2>&1
   rc=$?
   if [ $rc -eq 0 ]; then
@@ -166,8 +166,8 @@ check_using_gnu_ld () {
 
   printlabel _MKCONFIG_USING_GNU_LD "Using gnu ld"
 
-  # check for gcc...
-  ${CC} -v 2>&1 | grep 'GNU ld' > /dev/null 2>&1
+  # check for gnu ld...
+  ld -v 2>&1 | grep 'GNU ld' > /dev/null 2>&1
   rc=$?
   if [ $rc -eq 0 ]; then
       echo "found gnu ld" >&9
@@ -319,6 +319,38 @@ main () { return 0; }" > t.c
   setdata ${_MKCONFIG_PREFIX} CFLAGS "$ccflags ${flag}"
 }
 
+check_addldflag () {
+  name=$1
+  flag=$2
+  ccflags="${CFLAGS:-}"
+  ldflags="${LDFLAGS:-}"
+
+  printlabel LDFLAGS "Add LD flag: ${flag}"
+
+  echo "#include <stdio.h>
+main () { return 0; }" > t.c
+  echo "# test ${flag}" >&9
+  # need to set w/all cflags/ldflags; gcc doesn't always error out otherwise
+  TMPF=t$$.txt
+  ${CC} ${ccflags} ${ldflags} ${flag} -o t t.c > $TMPF 2>&1
+  rc=$?
+  if [ $rc -ne 0 ]; then
+    flag=0
+  fi
+  grep -i "warning.*${flag}" $TMPF > /dev/null 2>&1
+  rc=$?
+  if [ $rc -eq 0 ]; then
+    flag=0
+  fi
+  cat $TMPF >&9
+  rm -f $TMPF > /dev/null 2>&1
+  printyesno $name ${flag}
+  if [ $flag = 0 ]; then
+    flag=""
+  fi
+  setdata ${_MKCONFIG_PREFIX} LDFLAGS "$ldflags ${flag}"
+}
+
 check_ldflags () {
   _read_option LDFLAGS ""
   ldflags="${LDFLAGS:-${oval}}"
@@ -415,7 +447,7 @@ check_shcflags () {
   shcflags="-fPIC $SHCFLAGS"
   if [ "$_MKCONFIG_USING_GCC" != Y ]; then
     case ${_MKCONFIG_SYSTYPE} in
-      CYGWIN*)
+      CYGWIN*|MSYS*)
         shcflags="$SHCFLAGS"
         ;;
       Darwin)
@@ -586,3 +618,46 @@ check_sharerunpathflag () {
   printyesno_val SHRUNPATH "$SHRUNPATH"
   setdata ${_MKCONFIG_PREFIX} SHRUNPATH "$SHRUNPATH"
 }
+
+check_findincludepath () {
+  name=$1
+  hdr=$2
+  cincludes="${CFLAGS:-}"
+  printlabel CFLAGS "Search for: ${hdr}"
+  sp=""
+  incchk=""
+  pp=`echo $PATH | sed 's/:/ /g'`
+  set $pp
+  for p in $pp; do
+    case $p in
+      */bin)
+        td=`echo $p | sed 's,/bin$,/include,'`
+        if [ -d $td ]; then
+          if [ -f "$td/$hdr" ]; then
+            echo "found: ${td}" >&9
+            sp=$td
+            break
+          fi
+          list=`find $td -name ${hdr} -print | grep -v private 2>/dev/null`
+          for tp in $list; do
+            sp=`dirname $tp`
+            echo "find path:${sp}" >&9
+            break
+          done
+        fi
+        ;;
+    esac
+  done
+
+  echo "ccflags:${ccflags}" >&9
+  if [ "$sp" != "" ]; then
+    if [ $sp = "/usr/include" ]; then
+      printyesno_val $name "${hdr} yes"
+    else
+      printyesno_val $name "${hdr} -I${sp}"
+      setdata ${_MKCONFIG_PREFIX} CFLAGS "$ccflags -I$sp"
+    fi
+  else
+    printyesno_val $name "${hdr} no"
+  fi
+}
